#delimit ;

*! Author:
*! Hans-Peter Kohler
*! kohler@demogr.mpg.de
*! version 2.0 (August 20, 2000)
*! ;

program define twbiopr, eclass;
	version 6.0;
	if replay() {;
		if "`e(cmd)'" != "twbiopr" {;
			exit 301;
		};
		Replay `0';
	};
	else Estimate `0';
end;

program define Estimate, eclass;
version 6.0;
syntax varlist(min=2 max=2 numeric) 
       [if] [in] [aweight pweight] [, xbeqc(varlist)
				      xbeqi(string)
				      rhoeq(varlist)
                                      maxcat(integer -1)
                                      truncate
				      norhocons
				      rhogues(numlist) 
				      trace];

macro drop HP_*;

*! define dependent variables;
tokenize `varlist';
global HP_y1 `1';
global HP_y2 `2';

*! no. of vars in equation specifications;
local xbeqcw : word count `xbeqc';
local xbeqiw : word count `xbeqi';
global HP_xicn = `xbeqiw';
global HP_xiva   "`xbeqi'";
local rhoeqw : word count `rhoeq';

*! prepare macros for xbeqc variables;
if "`xbeqc'" != ""
  {;
   tokenize `xbeqc';
   local i = 1;
   while "``i''" != ""
     {;
      local xceqnam "`xceqnam' xbeqc:";
      local xcvanam "`xcvanam' ``i''";
      local i = `i' + 1;
    };
 };

*! confirm existence of individual specific variables;
*! prepare macros for ml call and ml evaluators;
if "`xbeqi'" != ""
  {;
   tokenize `xbeqi';
   local i = 1;
   while "``i''" != ""
     {;
      local xieqmlc "`xieqmlc' (``i'':)";
      local xieqnam "`xieqnam' ``i'':";
      local xivanam "`xivanam' _cons";
      local xieq1 "`xieq1' ``i''1";
      local xieq2 "`xieq2' ``i''2";
      global HP_xiec "$HP_xiec xiv`i'";
      local i = `i' + 1;
    };
 };


*! mark sample;    
tempvar touse;
mark `touse' `if' `in';
markout `touse' $HP_y1 $HP_y2 `xbeqc' `xieq1' `xieq2' `rhoeq';

*! set max category and trucate dep vars if requested;
if `maxcat' == -1 & "`truncat'" != ""
   {;
    di in red "need to specify option maxcat in order to use truncate";
    exit;
  };
if `maxcat' == -1
  {;
   qui su $HP_y1 if `touse';
   local maxdep1 = r(max);
   qui su $HP_y2 if `touse';
   local maxdep2 = r(max);
   local maxcat = max(`maxdep1',`maxdep1');
 };
if `maxcat' != -1 & "`truncat'" != ""
   {;
    preserve;
    qui replace $HP_y1 = `maxcat' if `touse' & $HP_y1 > `maxcat' & $HP_y1 != .;
    qui replace $HP_y2 = `maxcat' if `touse' & $HP_y2 > `maxcat' & $HP_y2 != .;
    di "Variables $HP_y1 and $HP_y2 have been trucated at `maxcat'";
    di "data will be restored after analysis";
  };
global HP_ncat = `maxcat' + 1;
global HP_ymax = `maxcat';


*! prepare macros for critical values;
local i = 1;
while `i' <= `maxcat' - 1
  {;
   local lceqmlc "`lceqmlc' (lnc`i':)";
   local lceqnam "`lceqnam' lnc`i':";
   local lcvanam "`lcvanam' _cons";
   global HP_lcec "$HP_lcec lnc`i'";
   local i = `i' + 1;
 };

*! prepare rho equation;
if "`rhocons'" != "" {;
   local rconst ", noconst";
   };
if "`rhocons'" == "" {;
   local rhoeqw = `rhoeqw' + 1;
   local rhoadn "_cons";
   };

*! various tests of command syntax;
if $HP_ncat > 6 {;
  di in red "more than 6 categories is not implemented";
  exit ;
};

*! inspect dependent variable;
local i = 1;
while `i' <= 2 {;
qui su ${HP_y`i'} if `touse';
if r(min) != 0 | r(max) != $HP_ymax {;
   di in red "dependent variable must contain integers between 0 and $HP_ymax";
   exit;
   };
qui inspect ${HP_y`i'} if `touse';
if r(N_unique) != $HP_ncat {;
   di in red "dependent variable must contain integers between 0 and $HP_ymax";
   exit;
   };
local i = `i' + 1;
};

*! incoprorate weights;
if "`weight'" != "" {;
   tempvar wvar;
   qui g double `wvar' `exp' if `touse';
   local weight "[`weight' = `wvar']";
   };

*! some temporary vars or names;
tempname b0 b1 b2 b3; 

*! bivariate probit (maxcat == 1);
if `maxcat' == 1
  {;
   qui probit $HP_y1 `xbeqc' `xieq1' `weight' if `touse';
   mat `b1' = e(b);
   mat coleq `b1' = `xceqnam' `xieqnam' xbeqc:;
   mat colnames `b1' = `xcvanam' `xivanam' _cons;
   if "`rhogues'" == ""
     {;
      local srch "on";
      mat `b3' = J(1,`rhoeqw',0);
    };
   else
     {;
      local srch "on";
      local rhoguw : word count `rhogues';
      if `rhoguw' == `rhoeqw'
        {;
         tokenize `rhogues';
         mat `b3' = `1';
         local k = 2;
         while `k' <= `rhoeqw'
           {;
	    mat `b3' = `b3',``k'';
	    local k = `k' + 1;
          };
       };
      else {;
           di in red "wrong number of arguments in rhogues";
           exit;
         };
    };
   mat coleq `b3' = rhoeq:;
   mat colnames `b3' = `rhoeq' `rhoadn';
   mat `b0' = (`b1',`b3');

   ml model lf twb2lf 
	    (xbeqc: `xbeqc')
            `xieqmlc'
	    (rhoeq: `rhoeq' `rconst') 
	    `weight' if `touse', 
		     max search(`srch')
		     title("Bivariate probit for twin analysis")
		     `score' `trace' 
		     init(`b0', skip);
 };

*! bivariate ordered probit;
if `maxcat' >= 2
  {;
   qui oprobit $HP_y1 `xbeqc' `xieq1' `weight' if `touse';
   mat `b1' = e(b);
   mat `b2' = `b1';
   mat coleq `b1' = `xceqnam' `xieqnam' xbeqc: `lceqnam';
   mat colnames `b1' = `xcvanam' `xivanam' _cons `lcvanam';
   local cutbeg = `xbeqcw' + `xbeqiw' + 1;
   local cutend = colsof(`b1');
   local k = `cutbeg';
   while `k' <= `cutend'
     {;
      local j = `k' - 1;
      if `k' == `cutbeg' {mat `b1'[1,`k'] = -1*`b2'[1,`k']};
      if `k' > `cutbeg' {;
	    mat `b1'[1,`k'] = log(`b2'[1,`k'] - `b2'[1,`j']);
	    };
      local k = `k' + 1;
    };
   if "`rhogues'" == ""
     {;
      local srch "on";
      mat `b3' = J(1,`rhoeqw',0);
    };
   else
     {;
      local srch "on";
      local rhoguw : word count `rhogues';
      if `rhoguw' == `rhoeqw'
        {;
         tokenize `rhogues';
         mat `b3' = `1';
         local k = 2;
         while `k' <= `rhoeqw'
           {;
	    mat `b3' = `b3',``k'';
	    local k = `k' + 1;
          };
       };
      else {;
           di in red "wrong number of arguments in rhogues";
           exit;
         };
    };
   mat coleq `b3' = rhoeq:;
   mat colnames `b3' = `rhoeq' `rhoadn';
   mat `b0' = (`b1',`b3');

   ml model lf twbolf 
	    (xbeqc: `xbeqc')
            `xieqmlc'
            `lceqmlc'
	    (rhoeq: `rhoeq' `rconst') 
	    `weight' if `touse', 
		     max search(`srch')
		     title("Bivariate ordered probit for twin analysis")
		     `score' `trace' 
		     init(`b0', skip);
 };

est local cmd "twbiopr";
est local ncat "$HP_ncat";
est local depvar "`varlist'";
est local xbeqc "`xbeqc'";
est local xbeqi1 "`xieq1'";
est local xbeqi2 "`xieq2'";
est local options = trim("`rhocons' `trace'");

Replay, `level';
end;

program define Replay;
version 6.0;
	syntax [, Level(int $S_level)];
di in gr
"*******************************************************************************";
di "";
di "BIVARIATE (ORDERED) PROBIT ESTIMATION FOR TWIN DATA:";
di "";
di in gr "dependent variables:            `e(depvar)'";
di in gr "method:                         `e(cmd)'";
di in gr "number of categories:           `e(ncat)'";
di in gr "tw-pair specific vars:          `e(xbeqc)'";
di in gr "twin 1 specific vars:           `e(xbeqi1)'";
di in gr "twin 2 specific vars:           `e(xbeqi2)'";
di in gr "options:                        `e(options)'";
di "";
ml display, level (`level');
end;





